using System;
using System.Globalization;
using System.Text;
using System.Text.RegularExpressions;
using MailBee.Mime;

namespace WebMailLite
{
	/// <summary>
	/// Summary description for Account.
	/// </summary>
	[Serializable]
	public class Account
	{
		#region Fields
		protected string _email;
		protected IncomingMailProtocol _mailProtocol;
		protected string _mailIncHost;
		protected string _mailIncLogin;
		protected string _mailIncPassword;
		protected int _mailIncPort;
		protected string _mailOutHost;
		protected string _mailOutLogin;
		protected string _mailOutPassword;
		protected int _mailOutPort;
		protected bool _mailOutAuthentication;
		protected short _horiz_resizer;
		#endregion

		#region Properties

		public string Email
		{
			get { return _email; }
			set { _email = value; }
		}

        public IncomingMailProtocol MailProtocol
        {
            get { return _mailProtocol; }
            set { _mailProtocol = value; }
        }

		public string MailIncomingHost
		{
			get { return _mailIncHost; }
			set { _mailIncHost = value; }
		}

		public string MailIncomingLogin
		{
			get { return _mailIncLogin; }
			set { _mailIncLogin = value; }
		}

		public string MailIncomingPassword
		{
			get { return _mailIncPassword; }
			set { _mailIncPassword = value; }
		}

		public int MailIncomingPort
		{
			get { return _mailIncPort; }
			set { _mailIncPort = value; }
		}

		public string MailOutgoingHost
		{
			get { return _mailOutHost; }
			set { _mailOutHost = value; }
		}

		public string MailOutgoingLogin
		{
			get { return _mailOutLogin; }
			set { _mailOutLogin = value; }
		}

		public string MailOutgoingPassword
		{
			get { return _mailOutPassword; }
			set { _mailOutPassword = value; }
		}

		public int MailOutgoingPort
		{
			get { return _mailOutPort; }
			set { _mailOutPort = value; }
		}

		public bool MailOutgoingAuthentication
		{
			get { return _mailOutAuthentication; }
			set { _mailOutAuthentication = value; }
		}

		public short HorizResizer
		{
			get { return _horiz_resizer; }
			set { _horiz_resizer = value; }
		}
		
		
		public bool IsDemo = false;
		
		#endregion

		public Account()
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
			_email = "";
			_mailProtocol = settings.IncomingMailProtocol;
			_mailIncHost = settings.IncomingMailServer;
			_mailIncLogin = "";
			_mailIncPassword = "";
			_mailIncPort = settings.IncomingMailPort;
			_mailOutHost = settings.OutgoingMailServer;
			_mailOutLogin = "";
			_mailOutPassword = "";
			_mailOutPort = settings.OutgoingMailPort;
			_mailOutAuthentication = settings.ReqSmtpAuth;
		}

		public static Account LoginAccount(string email, string login, string password)
		{
            WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
            return LoginAccount(email, login, password, settings.IncomingMailServer, settings.IncomingMailPort, settings.OutgoingMailServer, settings.OutgoingMailPort, settings.ReqSmtpAuth, false);
		}

		public static Account LoginAccount(string email, string login, string password, string incomingMailServer, int incomingPort, string outgoingMailServer, int outgoingMailPort, bool useSmtpAuthentication, bool advanced_login)
		{
            WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
            WebmailResourceManager resMan = (new WebmailResourceManagerCreator()).CreateResourceManager();

			Account acct = new Account();
			acct.Email = email;
			acct.MailIncomingLogin = login;
			acct.MailIncomingPassword = password;
			acct.MailIncomingHost = incomingMailServer;
			acct.MailIncomingPort = incomingPort;
			acct.MailOutgoingHost = outgoingMailServer;
			acct.MailOutgoingPort = outgoingMailPort;
			acct.MailOutgoingAuthentication = useSmtpAuthentication;

			if (settings.EnableWmServer)
			{
				bool userAuthenticated = false;
				string emailDomain = EmailAddress.GetDomainFromEmail(email);
				string emailUser = EmailAddress.GetAccountNameFromEmail(email);
				WMServerStorage storage = new WMServerStorage(acct);
				storage.ConnectAndLoginAdmin();
				string[] domains = storage.GetDomainList();
				foreach (string domain in domains)
				{
					if (string.Compare(emailDomain, domain, true, CultureInfo.InvariantCulture) == 0)
					{
						acct.MailProtocol = IncomingMailProtocol.WMServer;
						acct.MailOutgoingLogin = acct.Email;
						acct.MailOutgoingPassword = acct.MailIncomingPassword;
						acct.MailOutgoingHost = settings.WmServerHost;
						acct.MailOutgoingPort = settings.XMailSmtpPort;
						WMServerUser[] users = storage.GetUserList(emailDomain, emailUser);
						foreach (WMServerUser user in users)
						{
							if (string.Compare(emailUser, user.Name, true, CultureInfo.InvariantCulture) == 0)
							{
								if (string.Compare(acct.MailIncomingPassword, user.Password, true, CultureInfo.InvariantCulture) != 0)
								{
									throw new WebMailException(resMan.GetString("ErrorPOP3IMAP4Auth"));
								}
								if (string.Compare(user.Type, "U", StringComparison.InvariantCultureIgnoreCase) != 0)
								{
									throw new WebMailException(resMan.GetString("ErrorPOP3IMAP4Auth"));
								}
								userAuthenticated = true;
								break;
							}
						}
						break;
					}
				}
				if (!userAuthenticated) throw new WebMailException(resMan.GetString("ErrorPOP3IMAP4Auth"));
			}

			MailServerStorage mss = MailServerStorageCreator.CreateMailServerStorage(acct);
			try
			{
				mss.Connect(true);
			}
			catch (WebMailException)
			{
				throw;
			}
			finally
			{
				if (mss.IsConnected()) mss.Disconnect();
			}
			return acct;
		}

	}// END CLASS DEFINITION Account
}
