using System;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Text;
using System.Web;
using MailBee;
using MailBee.Mime;
using MailBee.Security;
using MailBee.SmtpMail;
using MailBeeSmtp = MailBee.SmtpMail;

namespace WebMailLite
{
	/// <summary>
	/// Summary description for Smtp.
	/// </summary>
	public class Smtp
	{
		// true if successfull, otherwise false
		public static void SendMail(Account account, MailMessage message)
		{
			WebmailResourceManager resMan = (new WebmailResourceManagerCreator()).CreateResourceManager();
			try
			{
				MailMessage messageToSend = new MailMessage();
				messageToSend.Priority = message.Priority;
				messageToSend.From = message.From;
				messageToSend.To = message.To;
				messageToSend.Cc = message.Cc;
				messageToSend.Bcc = message.Bcc;
				messageToSend.Subject = message.Subject;
				messageToSend.Date = DateTime.Now;
				messageToSend.BodyHtmlText = message.BodyHtmlText;
				messageToSend.BodyPlainText = message.BodyPlainText;

				
				
				string str = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"];
				messageToSend.Headers.Add("X-Originating-IP", str, false);

				foreach (Attachment attach in message.Attachments)
				{
					messageToSend.Attachments.Add(attach);
				}

				WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
				MailBee.SmtpMail.Smtp.LicenseKey = settings.LicenseKey;
				MailBee.SmtpMail.Smtp objSmtp = new MailBee.SmtpMail.Smtp();
				if (settings.EnableLogging)
				{
					objSmtp.Log.Enabled = true;
					string dataFolderPath = Utils.GetDataFolderPath();
					if (dataFolderPath != null)
					{
						objSmtp.Log.Filename = Path.Combine(dataFolderPath, Constants.logFilename);
					}
				}

				SmtpServer server = new SmtpServer();
				server.Name = ((account.MailOutgoingHost != null) && (account.MailOutgoingHost.Length > 0)) ? account.MailOutgoingHost : account.MailIncomingHost;
				server.Port = account.MailOutgoingPort;
				server.AccountName = ((account.MailOutgoingLogin != null) && (account.MailOutgoingLogin.Length > 0)) ? account.MailOutgoingLogin : account.MailIncomingLogin;
				server.Password = ((account.MailOutgoingPassword != null) && (account.MailOutgoingPassword.Length > 0)) ? account.MailOutgoingPassword : account.MailIncomingPassword;
				if (account.MailOutgoingAuthentication)
				{
					server.AuthMethods = AuthenticationMethods.Auto;
					server.AuthOptions = AuthenticationOptions.PreferSimpleMethods;
				}
				if (server.Port == 465)
				{
					server.SslMode = SslStartupMode.OnConnect;
					server.SslProtocol = SecurityProtocol.Auto;
					server.SslCertificates.AutoValidation = CertificateValidationFlags.None;
				}

				objSmtp.SmtpServers.Add(server);

				Encoding outgoingCharset = Utils.GetEncodingByCodePage(settings.DefaultUserCharset);
				messageToSend.Charset = outgoingCharset.HeaderName;

				objSmtp.Message = messageToSend;

				try
				{
					objSmtp.Connect();

					objSmtp.Send();
				}
				catch (MailBeeConnectionException)
				{
					throw new WebMailException(resMan.GetString("ErrorSMTPConnect"));
				}
				catch (MailBeeSmtpLoginBadCredentialsException)
				{
					throw new WebMailException(resMan.GetString("ErrorSMTPAuth"));
				}
				catch (MailBeeException ex)
				{
					throw new WebMailMailBeeException(ex);
				}
				finally
				{
					if (objSmtp.IsConnected) objSmtp.Disconnect();
				}
			}
			catch (MailBeeException ex)
			{
				throw new WebMailMailBeeException(ex);
			}
		}

	}// END CLASS DEFINITION Smtp
}
