using System;
using System.Collections;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.UI;
using System.Xml;
using System.Xml.XPath;
using MailBee;
using MailBee.Html;
using MailBee.ImapMail;
using MailBee.Mime;
using Attribute = MailBee.Html.TagAttribute;
using AttributeCollection = MailBee.Html.TagAttributeCollection;

namespace WebMailLite
{
	public class XmlPacketAttachment
	{
		public int size = 0;
		public bool inline = false;
		public string temp_name = string.Empty;
		public string name = string.Empty;
		public string mime_type = string.Empty;

		public static XmlPacketAttachment CreateWithXPath(XPathNodeIterator xpathIterator)
		{
			XmlPacketAttachment result = new XmlPacketAttachment();

			string attrValue = xpathIterator.Current.GetAttribute("size", "");
			if (attrValue.Length > 0) result.size = Convert.ToInt32(attrValue);

			attrValue = xpathIterator.Current.GetAttribute("inline", "");
			if (attrValue.Length > 0) result.inline = (string.Compare(attrValue, "1", true, CultureInfo.InvariantCulture) == 0) ? true : false;

			XPathNodeIterator tempNameIter = xpathIterator.Current.Select("temp_name");
			if (tempNameIter.MoveNext()) result.temp_name = Utils.DecodeHtml(tempNameIter.Current.Value);

			XPathNodeIterator nameIter = xpathIterator.Current.Select("name");
			if (nameIter.MoveNext()) result.name = Utils.DecodeHtml(nameIter.Current.Value);

			XPathNodeIterator mimeTypeIter = xpathIterator.Current.Select("mime_type");
			if (mimeTypeIter.MoveNext()) result.mime_type = Utils.DecodeHtml(mimeTypeIter.Current.Value);

			return result;
		}
	}

	public class XmlPacketMessages
	{
		public XmlPacketMessage[] messages;

		public XmlPacketMessages()
		{
			messages = new XmlPacketMessage[0];
		}

		public static XmlPacketMessages CreateWithXPath(XPathNodeIterator xpathIterator)
		{
			XmlPacketMessages result = null;
			XPathNodeIterator xpathMessagesIter = xpathIterator.Current.Select(@"messages");
			if (xpathMessagesIter.MoveNext())
			{
				result = new XmlPacketMessages();

				ArrayList messages = new ArrayList();
				XPathNodeIterator xpathMessageIter = xpathMessagesIter.Current.Select("message");
				while (xpathMessageIter.MoveNext())
				{
					XmlPacketMessage xmlMsg = XmlPacketMessage.CreateWithXPath(xpathMessageIter);
					if (xmlMsg != null)
					{
						messages.Add(xmlMsg);
					}
				}
				result.messages = (XmlPacketMessage[])messages.ToArray(typeof (XmlPacketMessage));
			}
			return result;
		}
	}

	public class XmlPacketMessage
	{
		public int priority = 3;
		public string from = string.Empty;
		public string to = string.Empty;
		public string cc = string.Empty;
		public string bcc = string.Empty;
		public string subject = string.Empty;
		public bool bodyIsHtml = false;
		public string bodyText = string.Empty;
		public XmlPacketAttachment[] attachments;

		public string uid = string.Empty;

		public static XmlPacketMessage CreateWithXPath(XPathNodeIterator xpathIterator)
		{
			XmlPacketMessage result = new XmlPacketMessage();

			string attrValue = xpathIterator.Current.GetAttribute("priority", "");
			if (attrValue.Length > 0) result.priority = Convert.ToInt32(attrValue);

			XPathNodeIterator uidIter = xpathIterator.Current.Select("uid");
			if (uidIter.MoveNext())
			{
				result.uid = Utils.DecodeHtml(uidIter.Current.Value);
			}
			else
			{
				result.uid = string.Empty;
			}

			XPathNodeIterator fromIter = xpathIterator.Current.Select("headers/from");
			if (fromIter.MoveNext()) result.from = Utils.DecodeHtml(fromIter.Current.Value);

			XPathNodeIterator toIter = xpathIterator.Current.Select("headers/to");
			if (toIter.MoveNext()) result.to = Utils.DecodeHtml(toIter.Current.Value);

			XPathNodeIterator ccIter = xpathIterator.Current.Select("headers/cc");
			if (ccIter.MoveNext()) result.cc = Utils.DecodeHtml(ccIter.Current.Value);

			XPathNodeIterator bccIter = xpathIterator.Current.Select("headers/bcc");
			if (bccIter.MoveNext()) result.bcc= Utils.DecodeHtml(bccIter.Current.Value);

			XPathNodeIterator subjectIter = xpathIterator.Current.Select("headers/subject");
			if (subjectIter.MoveNext()) result.subject = Utils.DecodeHtml(subjectIter.Current.Value);

			XPathNodeIterator bodyIter = xpathIterator.Current.Select("body");
			if (bodyIter.MoveNext())
			{
				attrValue = bodyIter.Current.GetAttribute("is_html", "");
				if (attrValue.Length > 0) result.bodyIsHtml = (string.Compare(attrValue, "1", true, CultureInfo.InvariantCulture) == 0) ? true : false;

				result.bodyText = bodyIter.Current.Value;
			}

			ArrayList attachments = new ArrayList();
			XPathNodeIterator attachmentIter = xpathIterator.Current.Select("attachments/attachment");
			while (attachmentIter.MoveNext())
			{
				XmlPacketAttachment attach = XmlPacketAttachment.CreateWithXPath(attachmentIter);
				if (attach != null)
				{
					attachments.Add(attach);
				}
			}
			result.attachments = (XmlPacketAttachment[])attachments.ToArray(typeof(XmlPacketAttachment));

			return result;
		}
	}

	public class XmlPacket
	{
		public string action;
		public string request;
		public Hashtable parameters;
		public XmlPacketMessages messages;
		public XmlPacketMessage message;

		public XmlPacket()
		{
			action = string.Empty;
			request = string.Empty;
            parameters = new Hashtable(StringComparer.OrdinalIgnoreCase);
            messages = null;
			message = null;
		}
	}


	/// <summary>
	/// Summary description for XmlPacketManager.
	/// </summary>
	public class XmlPacketManager
	{
		//private Account _acct;
		//private Page _webPage;
		private string _informationForUser;
		private BaseWebMailActions _actions;

		public Account CurrentAccount
		{
			get
			{
				if (_actions != null)
				{
					return _actions.CurrentAccount;
				}
				return null;
			}
			//set { _acct = value; }
		}

		public Page CurrentPage
		{
			get
			{
				if (_actions != null)
				{
					return _actions.CurrentPage;
				}
				return null;
			}
		}

		public XmlPacketManager(Account acct, Page webPage)
		{
			//_acct = acct;
			//_webPage = webPage;
			_actions = new BaseWebMailActions(acct, webPage);
		}

		public XmlPacket ParseClientXmlText(string xmlText)
		{
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xmlText);
			return ParseClientXmlDocument(doc);
		}

		public XmlPacket ParseClientXmlDocument(XmlDocument xmlDocument)
		{
			XmlPacket result = new XmlPacket();
			XmlNodeReader nodeReader = new XmlNodeReader(xmlDocument);

			XPathDocument xpathDoc = new XPathDocument(nodeReader);
			XPathNavigator xpathNav = xpathDoc.CreateNavigator();

			XPathNodeIterator xpathParamNodeIter = xpathNav.Select(@"webmail/param");
			while (xpathParamNodeIter.MoveNext())
			{
				string name = xpathParamNodeIter.Current.GetAttribute("name", "");
				string val = xpathParamNodeIter.Current.GetAttribute("value", "");
				if ((val == null) || (val.Length == 0))
				{
					val = Utils.DecodeHtml(xpathParamNodeIter.Current.Value);
				}

				switch (name.ToLower(CultureInfo.InvariantCulture))
				{
					case "action":
						result.action = val;
						break;
					case "request":
						result.request = val;
						break;
					default:
						result.parameters.Add(name, val);
						break;
				}
			}

			XPathNodeIterator webmailIter = xpathNav.Select("webmail");
			if (webmailIter.MoveNext())
			{
				result.messages = XmlPacketMessages.CreateWithXPath(webmailIter.Clone());

				XPathNodeIterator messageIter = webmailIter.Current.Select("message");
				if (messageIter.MoveNext())
				{
					result.message = XmlPacketMessage.CreateWithXPath(messageIter);
				}
			}

			return result;
		}

		public virtual XmlDocument CreateServerXmlDocumentResponse(XmlPacket clientPacket)
		{
			XmlDocument result = new XmlDocument();
			result.PreserveWhitespace = true;
			XmlDeclaration xmlDecl = result.CreateXmlDeclaration("1.0", "utf-8", "");
			result.AppendChild(xmlDecl);
				
			XmlElement webmailNode = result.CreateElement("webmail");
			result.AppendChild(webmailNode);
			WebmailResourceManager _resMan = (new WebmailResourceManagerCreator()).CreateResourceManager();
            WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
            try
			{
				if (string.Compare(clientPacket.action, "login", true, CultureInfo.InvariantCulture) == 0)
				{
					string email = Convert.ToString(clientPacket.parameters["email"]);
					string login = Convert.ToString(clientPacket.parameters["mail_inc_login"]);
					string password = Convert.ToString(clientPacket.parameters["mail_inc_pass"]);
					string inc_server = Convert.ToString(clientPacket.parameters["mail_inc_host"]);
					int inc_port = Convert.ToInt32(clientPacket.parameters["mail_inc_port"]);
					string out_server = Convert.ToString(clientPacket.parameters["mail_out_host"]);
					int out_port = Convert.ToInt32(clientPacket.parameters["mail_out_port"]);
					bool smtp_auth = (clientPacket.parameters["mail_out_auth"].ToString() == "1") ? true : false;
					bool advanced_login = (clientPacket.parameters["advanced_login"].ToString() == "1") ? true : false;

                    string aLFlag = "0";
                    if (advanced_login)
                    {
                        aLFlag = "1";
                    }
                    else
                    {
                        switch (settings.HideLoginMode)
                        {
                            case LoginMode.HideLoginFieldLoginIsAccount:
                                login = EmailAddress.Parse(email).GetAccountName();
                                break;
                            case LoginMode.HideLoginFieldLoginIsEmail:
                                login = email;
                                break;
                            case LoginMode.HideEmailField:
                            case LoginMode.HideEmailFieldDisplayDomainAfterLogin:
                                email = string.Format("{0}@{1}", login, settings.DefaultDomainOptional);
                                break;
                            case LoginMode.HideEmailFieldLoginIsLoginAndDomain:
                            case LoginMode.HideEmailFieldDisplayDomainAfterLoginAndLoginIsLoginAndDomain:
                                email = string.Format("{0}@{1}", login, settings.DefaultDomainOptional);
                                login = email;
                                break;
                        }
                    }
					if (Convert.ToInt32(clientPacket.parameters["hide_login"])<20) {
						if (Validation.CheckIt(Validation.ValidationTask.Email, email))
							email = Validation.Corrected;
						else
							throw (new WebMailException(_resMan.GetString(Validation.ErrorMessage)));
					}
					if (Validation.CheckIt(Validation.ValidationTask.Login, login, aLFlag))
						login = Validation.Corrected;
					else
						throw (new WebMailException(_resMan.GetString(Validation.ErrorMessage)));
					if (Validation.CheckIt(Validation.ValidationTask.Password, password))
						password = Validation.Corrected;
					else
						throw (new WebMailException(_resMan.GetString(Validation.ErrorMessage)));
					if (Validation.CheckIt(Validation.ValidationTask.INServer, inc_server, aLFlag))
						inc_server = Validation.Corrected;
					else
						throw (new WebMailException(_resMan.GetString(Validation.ErrorMessage)));
					if (Validation.CheckIt(Validation.ValidationTask.INPort, clientPacket.parameters["mail_inc_port"].ToString(), aLFlag))
						inc_port = int.Parse(Validation.Corrected);
					else
						throw (new WebMailException(_resMan.GetString(Validation.ErrorMessage)));
					if (Validation.CheckIt(Validation.ValidationTask.OUTServer, out_server, aLFlag))
						out_server = Validation.Corrected;
					else
						throw (new WebMailException(_resMan.GetString(Validation.ErrorMessage)));
					if (Validation.CheckIt(Validation.ValidationTask.OUTPort, clientPacket.parameters["mail_out_port"].ToString(), aLFlag))
						out_port = int.Parse(Validation.Corrected);
					else
						throw (new WebMailException(_resMan.GetString(Validation.ErrorMessage)));

					LoginAccount(webmailNode, email, login, password, inc_server, inc_port, out_server, out_port, smtp_auth, advanced_login);
				}

				if (CurrentAccount == null) throw new WebMailSessionException((new WebmailResourceManagerCreator()).CreateResourceManager().GetString("SessionIsEmpty"));

				switch (clientPacket.action.ToLower(CultureInfo.InvariantCulture))
				{
					case "get":
						switch (clientPacket.request.ToLower(CultureInfo.InvariantCulture))
						{
							case "account":
								GetAccount(webmailNode);
								break;
							case "message":
								GetMessage(webmailNode, Convert.ToString(clientPacket.parameters["uid"]), (MessageMode)Convert.ToInt32(clientPacket.parameters["mode"]), (clientPacket.parameters["charset"] != null) ? Convert.ToInt32(clientPacket.parameters["charset"]) : -1);
								break;
							case "messages":
								GetMessages(webmailNode, Convert.ToInt32(clientPacket.parameters["page"]));
								break;
							case "settings_list":
								GetSettingsList(webmailNode);
								break;
						}
						break;
					case "send":
						switch (clientPacket.request.ToLower(CultureInfo.InvariantCulture))
						{
							case "message":
								if (clientPacket.message != null)
								{
									SendMessage(webmailNode, clientPacket.message);
								}
								break;
						}
						break;
					case "operation_messages":
						if (clientPacket.messages != null)
						{
							GroupOperations(webmailNode, clientPacket.request, clientPacket.messages);
						}
						break;
				}

				if ((_informationForUser != null) && (_informationForUser.Length > 0))
				{
					XmlElement informationElement = result.CreateElement("information");
					informationElement.AppendChild(result.CreateCDataSection(Utils.EncodeHtml(_informationForUser)));
					webmailNode.AppendChild(informationElement);
					_informationForUser = string.Empty;
				}
			}
			catch (WebMailException ex)
			{
				Log.WriteException(ex);
				webmailNode.RemoveAll();
				if (ex is WebMailSessionException)
				{
					XmlElement errorElement = result.CreateElement("session_error");
					webmailNode.AppendChild(errorElement);
				}
				else
				{
					XmlElement errorElement = result.CreateElement("error");
					errorElement.AppendChild(result.CreateCDataSection(Utils.EncodeJsSaveString(Utils.EncodeHtml(ex.Message))));
					webmailNode.AppendChild(errorElement);
				}
			}
			catch (Exception ex)
			{
				Log.WriteException(ex);
				throw;
			}
			return result;
		}

		private void GetSettingsList(XmlElement webmailElement)
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();

			XmlNode settingsListElem = webmailElement.OwnerDocument.CreateElement("settings_list");

			XmlAttribute showTextLabelsAttr = webmailElement.OwnerDocument.CreateAttribute("show_text_labels");
			showTextLabelsAttr.Value = (settings.ShowTextLabels) ? "1" : "0";
			settingsListElem.Attributes.Append(showTextLabelsAttr);

			XmlAttribute msgsPerPageAttr = webmailElement.OwnerDocument.CreateAttribute("msgs_per_page");
			msgsPerPageAttr.Value = settings.MailsPerPage.ToString(CultureInfo.InvariantCulture);
			settingsListElem.Attributes.Append(msgsPerPageAttr);

			XmlAttribute mailboxLimitAttr = webmailElement.OwnerDocument.CreateAttribute("mailbox_limit");
			mailboxLimitAttr.Value = settings.MailboxSizeLimit.ToString(CultureInfo.InvariantCulture);
			settingsListElem.Attributes.Append(mailboxLimitAttr);

			XmlAttribute viewModeAttr = webmailElement.OwnerDocument.CreateAttribute("view_mode");
			viewModeAttr.Value = ((int)settings.Mode).ToString(CultureInfo.InvariantCulture);
			settingsListElem.Attributes.Append(viewModeAttr);

			XmlAttribute enableMailboxSizeLimitAttr = webmailElement.OwnerDocument.CreateAttribute("enable_mailbox_size_limit");
			enableMailboxSizeLimitAttr.Value = (settings.EnableMailboxSizeLimit) ? "1" : "0";
			settingsListElem.Attributes.Append(enableMailboxSizeLimitAttr);

			XmlElement defSkinElem = webmailElement.OwnerDocument.CreateElement("def_skin");
			defSkinElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(settings.DefaultSkin)));
			settingsListElem.AppendChild(defSkinElem);

			webmailElement.AppendChild(settingsListElem);
		}

		private MailMessage InitMailMessage(XmlPacketMessage message)
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
			MailMessage messageToSend = new MailMessage();
			messageToSend.Priority = (MailPriority)message.priority;
			messageToSend.From = EmailAddress.Parse(Utils.DecodeHtml(message.from));
			messageToSend.To = EmailAddressCollection.Parse(Utils.DecodeHtml(message.to));
			messageToSend.Cc = EmailAddressCollection.Parse(Utils.DecodeHtml(message.cc));
			messageToSend.Bcc = EmailAddressCollection.Parse(Utils.DecodeHtml(message.bcc));
			messageToSend.Subject = Utils.DecodeHtml(message.subject);
			messageToSend.Date = DateTime.Now;
			messageToSend.Charset = Utils.GetEncodingByCodePage(settings.DefaultUserCharset).HeaderName;
			if (message.bodyIsHtml)
			{
				Processor htmlProcessor = new Processor();
				htmlProcessor.Dom.OuterHtml = Utils.DecodeHtmlBody(message.bodyText);
				ElementCollection images = htmlProcessor.Images;
				RuleSet rs = new RuleSet();
				foreach (XmlPacketAttachment xmlAttach in message.attachments)
				{
					if (!xmlAttach.inline) continue;

					foreach (Element imageElement in images)
					{
						AttributeCollection searchAttrs = new AttributeCollection();
						AttributeCollection addAttrs = new AttributeCollection();
						AttributeCollection removeAttrs = new AttributeCollection();

						foreach (MailBee.Html.TagAttribute attr in imageElement.Attributes)
						{
							if (attr.Name.ToLower() == "src")
							{
								string cid = string.Empty;
								byte[] buffer = null;
								string filename = string.Empty;
								Regex r = new Regex(@"filename=(?<filename>[\w\.]+)", RegexOptions.None);
								Match m = r.Match(attr.Value);
								if (m.Success)
								{
									filename = m.Groups["filename"].Value;
									string tempFolder = Utils.GetTempFolderName((CurrentPage != null) ? CurrentPage.Session : null);
									string fullName = Path.Combine(tempFolder, filename);
									if (File.Exists(fullName))
									{
										using (FileStream fs = File.OpenRead(fullName))
										{
											buffer = new byte[fs.Length];
											fs.Read(buffer, 0, buffer.Length);						
										}
										if (buffer != null)
										{
											cid = Path.GetFileNameWithoutExtension(filename);
											messageToSend.Attachments.Add(buffer, xmlAttach.name, cid, null, null, NewAttachmentOptions.Inline, MailTransferEncoding.Base64);
										}
									}
								}

								MailBee.Html.TagAttribute searchAttr = new Attribute();
								searchAttr.Name = attr.Name;
								searchAttr.Value = Regex.Escape(attr.Value);

								searchAttrs.Add(searchAttr);

								MailBee.Html.TagAttribute oldAttr = new Attribute();
								oldAttr.Definition = attr.Name;
								removeAttrs.Add(oldAttr);

								MailBee.Html.TagAttribute newAttr = new Attribute();
								//newAttr.Definition = "src='about:blank'";
								newAttr.Name = attr.Name;
								newAttr.Value = string.Format(@"""cid:{0}""", cid);
								addAttrs.Add(newAttr);

								break;
							}
						}
						rs.AddTagProcessingRule("img", searchAttrs, addAttrs, searchAttrs, true);
					}
				}
				messageToSend.BodyHtmlText = htmlProcessor.Dom.ProcessToString(rs, null);
			}
			else
			{
				messageToSend.BodyPlainText = Utils.DecodeHtmlBody(message.bodyText);
			}
			foreach (XmlPacketAttachment xmlAttach in message.attachments)
			{
				if (!xmlAttach.inline)
				{
					string fullPath = Path.Combine(Utils.GetTempFolderName((CurrentPage != null) ? CurrentPage.Session : null), Utils.DecodeHtml(xmlAttach.temp_name));
					messageToSend.Attachments.Add(fullPath, Utils.DecodeHtml(xmlAttach.name));
				}
			}

			return messageToSend;
		}

		private void SendMessage(XmlElement webmailNode, XmlPacketMessage message)
		{
            MailMessage msg = InitMailMessage(message);

			_actions.SendMessage(msg);

			XmlElement updateElem = webmailNode.OwnerDocument.CreateElement("update");

			XmlAttribute valueAttr = webmailNode.OwnerDocument.CreateAttribute("value");
            valueAttr.Value = "send_message";
            updateElem.Attributes.Append(valueAttr);

			webmailNode.AppendChild(updateElem);
		}

		private void GetAccount(XmlElement webmailNode)
		{
			if ((HttpContext.Current != null) && (HttpContext.Current.Session != null))
			{
				Account acct = HttpContext.Current.Session[Constants.sessionAccount] as Account;
				if (acct != null)
				{
					XmlElement accountElem = webmailNode.OwnerDocument.CreateElement("account");

					XmlElement emailElem = webmailNode.OwnerDocument.CreateElement("email");
					emailElem.AppendChild(webmailNode.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(acct.Email)));
					accountElem.AppendChild(emailElem);

					webmailNode.AppendChild(accountElem);
				}
			}
			else
			{
				throw new WebMailSessionException((new WebmailResourceManagerCreator()).CreateResourceManager().GetString("SessionIsEmpty"));
			}
		}

		private void GetMessages(XmlElement webmailElement, int page)
		{
			int inboxMessageCount = 0;
			long inboxSize = 0;
			MailMessageCollection msgsColl =_actions.GetMessages(page, out inboxMessageCount, out inboxSize);

			XmlNode messagesElem = webmailElement.OwnerDocument.CreateElement("messages");

			XmlAttribute pageAttr = webmailElement.OwnerDocument.CreateAttribute("page");
			pageAttr.Value = page.ToString();
			messagesElem.Attributes.Append(pageAttr);

			XmlAttribute countAttr = webmailElement.OwnerDocument.CreateAttribute("count");
			countAttr.Value = inboxMessageCount.ToString();
			messagesElem.Attributes.Append(countAttr);

			XmlAttribute mailboxSizeAttr = webmailElement.OwnerDocument.CreateAttribute("mailbox_size");
			mailboxSizeAttr.Value = inboxSize.ToString(CultureInfo.InvariantCulture);
			messagesElem.Attributes.Append(mailboxSizeAttr);

			if (msgsColl != null)
			{
				foreach (MailMessage msg in msgsColl)
				{
					XmlNode messageElem = webmailElement.OwnerDocument.CreateElement("message");

					XmlAttribute hasAttachmentsAttr = webmailElement.OwnerDocument.CreateAttribute("has_attachments");
					hasAttachmentsAttr.Value = (msg.HasAttachments) ? "1" : "0";
					messageElem.Attributes.Append(hasAttachmentsAttr);

					XmlAttribute priorityAttr = webmailElement.OwnerDocument.CreateAttribute("priority");
					int priority = 3;
					switch (msg.Priority)
					{
						case MailPriority.High:
						case MailPriority.Highest:
							priority = 1;
							break;
						case MailPriority.Low:
						case MailPriority.Lowest:
							priority = 5;
							break;
						case MailPriority.Normal:
							priority = 3;
							break;
					}
					priorityAttr.Value = priority.ToString(CultureInfo.InvariantCulture);
					messageElem.Attributes.Append(priorityAttr);

					XmlAttribute sizeAttr = webmailElement.OwnerDocument.CreateAttribute("size");
					string sizeStr = "0";
					if (_actions.CurrentAccount != null)
					{
						sizeStr = (_actions.CurrentAccount.MailProtocol != IncomingMailProtocol.WMServer) ? msg.SizeOnServer.ToString(CultureInfo.InvariantCulture) : ((XMailMessage)msg).SizeXMail.ToString(CultureInfo.InvariantCulture);
					}

					sizeAttr.Value = sizeStr; 
					messageElem.Attributes.Append(sizeAttr);

					string from = ((msg.From.DisplayName != null) && (msg.From.DisplayName.Length > 0)) ? msg.From.DisplayName : msg.From.ToString();
					XmlElement fromElem = webmailElement.OwnerDocument.CreateElement("from");
					fromElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(from)));
					messageElem.AppendChild(fromElem);

					XmlElement toElem = webmailElement.OwnerDocument.CreateElement("to");
					toElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(Utils.GetAddressesFriendlyName(msg.To))));
					messageElem.AppendChild(toElem);

					EmailAddressCollection replyTo = (msg != null) ? msg.ReplyTo : new EmailAddressCollection(string.Empty);
					XmlElement replyToElem = webmailElement.OwnerDocument.CreateElement("reply_to");
					toElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(Utils.GetAddressesFriendlyName(replyTo))));
					messageElem.AppendChild(replyToElem);

					XmlElement ccElem = webmailElement.OwnerDocument.CreateElement("cc");
					toElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(Utils.GetAddressesFriendlyName(msg.Cc))));
					messageElem.AppendChild(ccElem);

					XmlElement bccElem = webmailElement.OwnerDocument.CreateElement("bcc");
					toElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(Utils.GetAddressesFriendlyName(msg.Bcc))));
					messageElem.AppendChild(bccElem);

					XmlElement subjectElem = webmailElement.OwnerDocument.CreateElement("subject");
					subjectElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(msg.Subject)));
					messageElem.AppendChild(subjectElem);

					XmlElement dateElem = webmailElement.OwnerDocument.CreateElement("date");
					dateElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(Utils.GetDateWithOffsetFormatString(CurrentAccount, msg.DateReceived.ToUniversalTime()))));
					messageElem.AppendChild(dateElem);

					XmlElement uidElem = webmailElement.OwnerDocument.CreateElement("uid");
					string uid = (_actions.CurrentAccount.MailProtocol != IncomingMailProtocol.WMServer) ? msg.UidOnServer.ToString() : ((XMailMessage)msg).UidXMail;
					uidElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(uid)));
					messageElem.AppendChild(uidElem);

					messagesElem.AppendChild(messageElem);
				}
			}
			webmailElement.AppendChild(messagesElem);
		}

		private void GetMessage(XmlElement webmailElement, string uid, MessageMode mode, int charset)
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();

			byte safety = 0;
			bool showPicturesSettings = ((settings.Mode & ViewMode.AlwaysShowPictures) > 0) ? true : false;
			if (showPicturesSettings) safety = 1;
			MailMessage msg = _actions.GetMessage(uid, charset, safety);

			if (msg != null)
			{
				XmlNode messageElem = webmailElement.OwnerDocument.CreateElement("message");

				XmlElement uidElem = webmailElement.OwnerDocument.CreateElement("uid");
				uidElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(uid.ToString(CultureInfo.InvariantCulture))));
				messageElem.AppendChild(uidElem);

				bool hasHtml = false;
				if (msg != null)
				{
					hasHtml = ((msg.BodyHtmlText != null) && (msg.BodyHtmlText.Length > 0)) ? true : false;
				}
				XmlAttribute attrNode = webmailElement.OwnerDocument.CreateAttribute("html");
				attrNode.Value = (hasHtml) ? "1" : "0";
				messageElem.Attributes.Append(attrNode);

				bool hasPlain = false;
				if (msg != null)
				{
					hasPlain = ((msg.BodyPlainText != null) && (msg.BodyPlainText.Length > 0)) ? true : false;
				}
				attrNode = webmailElement.OwnerDocument.CreateAttribute("plain");
				attrNode.Value = (hasPlain) ? "1" : "0";
				messageElem.Attributes.Append(attrNode);

				int priority = 3;
				switch (msg.Priority)
				{
					case MailPriority.High:
					case MailPriority.Highest:
						priority = 1;
						break;
					case MailPriority.Low:
					case MailPriority.Lowest:
						priority = 5;
						break;
					case MailPriority.Normal:
						priority = 3;
						break;
				}
				attrNode = webmailElement.OwnerDocument.CreateAttribute("priority");
				attrNode.Value = priority.ToString(CultureInfo.InvariantCulture);
				messageElem.Attributes.Append(attrNode);

				bool hasCharset = false;
				if (msg != null)
				{
					if (msg.Charset.Length != 0) hasCharset = true;
				}
				attrNode = webmailElement.OwnerDocument.CreateAttribute("has_charset");
				attrNode.Value = (hasCharset) ? "1" : "0";
				messageElem.Attributes.Append(attrNode);

				MessageMode newMode = mode;
				if ((mode & MessageMode.Headers) > 0)
				{
					newMode |= MessageMode.Headers;
					XmlNode headersNode = webmailElement.OwnerDocument.CreateElement("headers");

					XmlNode fromNode = webmailElement.OwnerDocument.CreateElement("from");
					fromNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(msg.From.ToString())));
					headersNode.AppendChild(fromNode);

					XmlNode toNode = webmailElement.OwnerDocument.CreateElement("to");
					toNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(msg.To.ToString())));
					headersNode.AppendChild(toNode);

					XmlNode ccNode = webmailElement.OwnerDocument.CreateElement("cc");
					ccNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(msg.Cc.ToString())));
					headersNode.AppendChild(ccNode);

					XmlNode bccNode = webmailElement.OwnerDocument.CreateElement("bcc");
					bccNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(msg.Bcc.ToString())));
					headersNode.AppendChild(bccNode);

					XmlNode replyToNode = webmailElement.OwnerDocument.CreateElement("reply_to");
					replyToNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(msg.ReplyTo.ToString())));
					headersNode.AppendChild(replyToNode);

					XmlNode subjectNode = webmailElement.OwnerDocument.CreateElement("subject");
					subjectNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(msg.Subject.ToString())));
					headersNode.AppendChild(subjectNode);

					XmlNode dateNode = webmailElement.OwnerDocument.CreateElement("date");
					DateTime dt = Constants.MinDate;
					try
					{
						dt = (msg.DateReceived != DateTime.MinValue) ? msg.DateReceived : msg.Date;
					}
					catch (MailBeeDateParsingException)
					{
						dt = DateTime.Now;
					}
					dateNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(Utils.GetDateWithOffsetFormatString(CurrentAccount, dt.ToUniversalTime()))));
					headersNode.AppendChild(dateNode);

					messageElem.AppendChild(headersNode);
				}

				if ((mode & MessageMode.HtmlBody) > 0)
				{
					if (msg != null)
					{
						if ((msg.BodyHtmlText != null) && (msg.BodyHtmlText.Length > 0))
						{
							newMode |= MessageMode.HtmlBody;
							XmlNode htmlPartNode = webmailElement.OwnerDocument.CreateElement("html_part");
							htmlPartNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeCDATABody(msg.BodyHtmlText)));
							messageElem.AppendChild(htmlPartNode);
						}
						else
						{
							newMode |= MessageMode.PlainBody; // if we haven't HTML, return Plain
						}
					}
				}

				if ((mode & MessageMode.PlainBody) > 0 || (newMode & MessageMode.PlainBody) > 0)
				{
                    string modifiedPlainText = Utils.MakeHtmlBodyFromPlainBody(msg.BodyPlainText);
                    newMode |= MessageMode.PlainBody;
					XmlNode modifiedPlainTextNode = webmailElement.OwnerDocument.CreateElement("modified_plain_text");
                    modifiedPlainTextNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeCDATABody(modifiedPlainText)));
					messageElem.AppendChild(modifiedPlainTextNode);
				}

				if (((mode & MessageMode.HtmlReply) > 0) || ((mode & MessageMode.HtmlForward) > 0))
				{
					if (msg != null)
					{
						string mess = Utils.GetMessageHtmlReplyToBody(CurrentAccount, msg);
						if ((mode & MessageMode.HtmlReply) > 0)
						{
							newMode |= MessageMode.HtmlReply;
							XmlElement replyHtmlElem = webmailElement.OwnerDocument.CreateElement("reply_html");
							replyHtmlElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeCDATABody(mess)));
							messageElem.AppendChild(replyHtmlElem);
						}
						if ((mode & MessageMode.HtmlForward) > 0)
						{
							newMode |= MessageMode.HtmlForward;
							XmlElement forwardHtmlElem = webmailElement.OwnerDocument.CreateElement("forward_html");
							forwardHtmlElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeCDATABody(mess)));
							messageElem.AppendChild(forwardHtmlElem);
						}
					}
				}

				if (((mode & MessageMode.PlainReply) > 0) || ((mode & MessageMode.PlainForward) > 0))
				{
					if (msg != null)
					{
						string mess = Utils.GetMessagePlainReplyToBody(CurrentAccount, msg);
						mess = Utils.DecodeHtml(mess);
						mess = mess.Replace("<br>"," ");
						mess = Utils.RemoveATag(mess);
						if ((mode & MessageMode.PlainReply) > 0)
						{
							newMode |= MessageMode.PlainReply;
							XmlElement replyPlainElem = webmailElement.OwnerDocument.CreateElement("reply_plain");
							replyPlainElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeCDATABody(mess)));
							messageElem.AppendChild(replyPlainElem);
						}
						if ((mode & MessageMode.PlainForward) > 0)
						{
							newMode |= MessageMode.PlainForward;
							XmlElement forwardPlainElem = webmailElement.OwnerDocument.CreateElement("forward_plain");
							forwardPlainElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeCDATABody(mess)));
							messageElem.AppendChild(forwardPlainElem);
						}
					}
				}

				if ((mode & MessageMode.FullHeaders) > 0)
				{
					newMode |= MessageMode.FullHeaders;
					XmlNode fullHeadersNode = webmailElement.OwnerDocument.CreateElement("full_headers");
                    fullHeadersNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection((msg != null) ? Utils.EncodeCDATABody(msg.RawHeader) : string.Empty));
					messageElem.AppendChild(fullHeadersNode);
				}

				if ((mode & MessageMode.Attachments) > 0)
				{
					if (msg.Attachments.Count > 0)
					{
						newMode |= MessageMode.Attachments;
						XmlNode attachmntsNode = webmailElement.OwnerDocument.CreateElement("attachments");
						if (msg != null)
						{
							foreach (Attachment attach in msg.Attachments)
							{
								string attachmentName = (attach.Filename.Length > 0) ? attach.Filename : attach.Name;

								XmlNode attachmentNode = webmailElement.OwnerDocument.CreateElement("attachment");

								attrNode = webmailElement.OwnerDocument.CreateAttribute("size");
								attrNode.Value = attach.Size.ToString();
								attachmentNode.Attributes.Append(attrNode);

								attrNode = webmailElement.OwnerDocument.CreateAttribute("inline");
								attrNode.Value = (attach.IsInline) ? "1" : "0";
								attachmentNode.Attributes.Append(attrNode);

								XmlElement filenameElem = webmailElement.OwnerDocument.CreateElement("filename");
								filenameElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(attachmentName));
								attachmentNode.AppendChild(filenameElem);

								string filename = Path.GetFileName(attach.SavedAs);
								if (attach.ContentType.ToLower().StartsWith("image"))
								{
									XmlNode viewNode = webmailElement.OwnerDocument.CreateElement("view");
									viewNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.GetAttachmentDownloadLink(attach, true)));
									attachmentNode.AppendChild(viewNode);
								}

								XmlNode downloadNode = webmailElement.OwnerDocument.CreateElement("download");
								downloadNode.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.GetAttachmentDownloadLink(attach, false)));
								attachmentNode.AppendChild(downloadNode);

								XmlElement tempnameElem = webmailElement.OwnerDocument.CreateElement("tempname");
								tempnameElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(filename)));
								attachmentNode.AppendChild(tempnameElem);

								XmlElement mimeTypeElem = webmailElement.OwnerDocument.CreateElement("mime_type");
								mimeTypeElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.EncodeHtml(Utils.GetAttachmentMimeTypeFromFileExtension(Path.GetExtension(filename)))));
								attachmentNode.AppendChild(mimeTypeElem);

								attachmntsNode.AppendChild(attachmentNode);
							}
						}
						messageElem.AppendChild(attachmntsNode);
					}
				}
				attrNode = webmailElement.OwnerDocument.CreateAttribute("mode");
				attrNode.Value = ((int)newMode).ToString(CultureInfo.InvariantCulture);
				messageElem.Attributes.Append(attrNode);

				attrNode = webmailElement.OwnerDocument.CreateAttribute("charset");
				attrNode.Value = charset.ToString(CultureInfo.InvariantCulture);
				messageElem.Attributes.Append(attrNode);

				XmlElement saveLinkElem = webmailElement.OwnerDocument.CreateElement("save_link");
				string messageUid = (_actions.CurrentAccount.MailProtocol != IncomingMailProtocol.WMServer) ? msg.UidOnServer.ToString() : ((XMailMessage)msg).UidXMail;
				saveLinkElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.GetMessageDownloadLink(messageUid)));
				messageElem.AppendChild(saveLinkElem);

				XmlElement printLinkElem = webmailElement.OwnerDocument.CreateElement("print_link");
				printLinkElem.AppendChild(webmailElement.OwnerDocument.CreateCDataSection(Utils.GetMessagePrintLink(msg)));
				messageElem.AppendChild(printLinkElem);

				webmailElement.AppendChild(messageElem);
			}
			else
			{
				throw new WebMailException("Message not exists.");
			}
		}

		private void LoginAccount(XmlElement webmailElement, string email, string login, string password, string inc_server, int inc_port, string out_server, int out_port, bool smtp_auth, bool advanced_login)
		{
			_actions.LoginAccount(email, login, password, inc_server, inc_port, out_server, out_port, smtp_auth, advanced_login);
			XmlElement loginElem = webmailElement.OwnerDocument.CreateElement("login");
			webmailElement.AppendChild(loginElem);
		}

		private void GroupOperations(XmlElement webmailElement, string request, XmlPacketMessages messages)
		{
			ArrayList uids = new ArrayList();
			foreach (XmlPacketMessage message in messages.messages)
			{
				uids.Add(message.uid);
			}
			_actions.DeleteMessages((string[])uids.ToArray(typeof(string)));

			XmlElement operationMessagesElement = webmailElement.OwnerDocument.CreateElement("operation_messages");
			
			XmlAttribute attributeNode = webmailElement.OwnerDocument.CreateAttribute("type");
			attributeNode.Value = request;
			operationMessagesElement.Attributes.Append(attributeNode);

			webmailElement.AppendChild(operationMessagesElement);
		}

	}
}
