using System;
using System.Collections.Specialized;

namespace WebMailLite.classic
{

	/// <summary>
	/// Summary description for jsbuilder.
	/// </summary>
	public class jsbuilder
	{
		const string CRLF = "\r\n";
		private StringCollection _jsFiles = new StringCollection();
		private StringCollection _jsText = new StringCollection();
		private StringCollection _jsInitFunctionText = new StringCollection();

		public jsbuilder()
		{
			this.AddFile("_defines.js");
			this.AddFile("langs_js.aspx");
			this.AddFile("_functions.js");
			this.AddFile("class.common.js");
			this.AddFile("class.popup-menu.js");
		}

		public void AddFile(string text)
		{
			text = text.Trim();
			if (text.Length > 0)
			{
				if (!this._jsFiles.Contains(text))
				{
					this._jsFiles.Add(text);
				}
			}
		}

		public void AddText(string text)
		{
			text = text.Trim();
			if (text.Length > 0)
			{
				this._jsText.Add(text);
			}
		}

		public void AddInitText(string text)
		{
			text = text.Trim();
			if (text.Length > 0)
			{
				this._jsInitFunctionText.Add(text);
			}
		}
		
		public string FilesToHtml()
		{
			string returString = "";
			foreach (string fileString in this._jsFiles)
			{
				returString += string.Format(@"<script language=""JavaScript"" type=""text/javascript"" src=""{0}""></script>" + jsbuilder.CRLF, fileString);
			}
			return returString;
		}

		public string TextToHtml()
		{
			string returString = "";
			if (this._jsText.Count > 0)
			{
				returString += @"<script type=""text/javascript"">" + jsbuilder.CRLF;
				foreach (string textString in this._jsText)
				{
					returString += textString + jsbuilder.CRLF;
				}
				returString += jsbuilder.CRLF + "</script>" + jsbuilder.CRLF;
			}
			return returString;
		}

		public string InitTextToHtml()
		{
			string returString = @"<script type=""text/javascript"">
function Init()
{
	Browser = new CBrowser();
	PopupMenu = new CPopupMenus();
";
			foreach (string initString in this._jsInitFunctionText)
			{
				returString += initString + jsbuilder.CRLF;
			}

			returString += @"ResizeElements(""all"");
}
</script>";
			
			return returString;
		}

		public string ReadyToHtml()
		{
			string returString = @"
<script type=""text/javascript"">
	var INIT_DEFINES     = 0;
	var INIT_COMMON      = 1;
	var INIT_FUNCTIONS   = 2;
	var INIT_BODY        = 3;

	var ReadyInit = Array();
	ReadyInit[INIT_DEFINES]     = false;
	ReadyInit[INIT_COMMON]      = false;
	ReadyInit[INIT_FUNCTIONS]   = false;
	ReadyInit[INIT_BODY]        = false;

	function Ready(fileId)
	{
		ReadyInit[fileId] = true;
		var isReady = true;
		for (var i=0; i<=INIT_BODY; i++)
		{
			if (ReadyInit[i] == false)
			{
				isReady = false;
				break;
			}
		}
		if (isReady) Init();
	}
</script>
			";
			return returString;
		}

		public string ToHtml()
		{
			return jsbuilder.CRLF + this.ReadyToHtml() + this.FilesToHtml() + this.TextToHtml();
		}

	}
}